##########################################
## Cosmetics Europe Skin Sensitization
## Defined Approach Evaluation
## Nicole Kleinstreuer, Jason Pirone
## nicole.kleinstreuer@nih.gov
##
## Shiseido ANN model
##
## Hirota et al. 2015 (143 chems)
##
############################################ 

rm(list=ls())

##Change directory string
input.dir <- "C:/Users/Nicole/Documents/Cosmetics Europe/"
date.string<-Sys.Date()
setwd(input.dir)

library(gdata)
library(caret)
library(e1071)

##Change directory string
code.dir <- "C:/Users/Nicole/Documents/Local Code folder/Cosmetics Europe code/Final Code"
##Source code for ANN functions
source(paste(code.dir,"5DA_Shiseido_ANN_functions.r",sep=""))

##########################################
##Read Cosmetic Europe in vitro data, physchem properties, OECD Toolbox predictions
CE<-read.csv("CosEU_invitro_insilico_128chems_Jan2017.csv",header=TRUE,stringsAsFactors=FALSE)
CE$CASRN<-trim(CE$CASRN)
CE<-CE[order(CE$Num),]
cosEu<-CE
##Format data: replace all negative DPRA values with 0
cosEu$DPRA.mean[cosEu$DPRA.mean<0]<-0
##Remove chemicals without in vitro data 
cosEu <- cosEu[-which(is.na(cosEu$DPRA.Call) | is.na(cosEu$hCLAT.Call)),]

##########################################
##Read data from Hirota et al. 2015 ANN models
hirota15 <- read.csv("Hirota_2015_Updated_Clean_092515.csv",stringsAsFactors=FALSE)
rownames(hirota15) <- hirota15$CASRN

##########################################
##Read in vivo LLNA and human data
cosEuNegCons <- read.csv("CosEu_invivo_128chems_March2017.csv",stringsAsFactors=FALSE,na.strings=c("","NA","NC"))
##Force all negative LLNA EC3 values to 100 (log EC3=2)
cosEuNegCons$Median_NC.EC3[cosEuNegCons$LLNA.Call.MNC==0]<-100
##Fix formatting in column names
colnames(cosEuNegCons) <- gsub("\\.$", "", colnames(cosEuNegCons))
colnames(cosEuNegCons) <- gsub("(\\.)+", "_", colnames(cosEuNegCons))
##Check for consistency and merge in vitro/in vivo data
all(cosEuNegCons$Num %in% cosEu$Num)
cosEu <- merge(cosEu, cosEuNegCons, by="Num", sort=FALSE)
colnames(cosEu) <- gsub("CASRN.x","CASRN", colnames(cosEu))
rownames(cosEu) <- cosEu$CASRN
##Log transform LLNA values (Replace Median_NC_EC3 with alternate metric as needed)
cosEu$logLLNAthresholdMedNegCons <- log10(cosEu$Median_NC_EC3)
cosEu$logLLNAthresholdMedNegCons[is.na(cosEu$logLLNAthresholdMedNegCons)] <- cosEu$logLLNAthreshold[is.na(cosEu$logLLNAthresholdMedNegCons)]

##Check for overlap
cosEu$InHirota <- cosEu$CASRN %in% hirota15$CASRN

##Applying logic to deal with negatives (If all model inputs are negative, prediction is negative)
cosEu$ANN_Model1_Pred <- "Pos"
cosEu$Pos.Call.DH<-rowSums(cosEu[,c("DPRA.Call","hCLAT.Call")])
cosEu$ANN_Model1_Pred[cosEu$Pos.Call.DH==0] <- "Neg"
cosEu$ANN_Model4_Pred <- "Pos"
cosEu$Pos.Call.DHK<-rowSums(cosEu[,c("DPRA.Call","hCLAT.Call","KS.Call")])
cosEu$ANN_Model4_Pred[cosEu$Pos.Call.DHK==0] <- "Neg"

##Replace negatives in hCLAT with CV75
cosEu$hCLAT.MIT[cosEu$hCLAT.MIT=="Inf"] <- cosEu$h.CLAT.CV75[cosEu$hCLAT.MIT=="Inf"]
cosEu$logMinhCLAT <- log10(cosEu$hCLAT.MIT)
cosEu$logMinhCLAT[cosEu$logMinhCLAT=="Inf"]<-NA
##Replace 0 in DPRA with 0.01
cosEu$DPRA.mean[cosEu$DPRA.mean==0]<-0.01
cosEu$logDPRA <- log10(cosEu$DPRA.mean)
##Format KS data
cosEu$logImax <- log10(cosEu$KS.Imax)

#############################################################################
##Define inputs/targets for respective models and number of training iterations
nnTarget <- "logLLNAthresholdMedNegCons"
model1Input <- c("logMinhCLAT", "logDPRA")
model4Input <- c("logMinhCLAT", "logDPRA", "logImax")
trainIters <- 10000

#############################################################################
##Model 1 Training (139 chemicals from Hirota et al. 2015)
X1 <- hirota15[, model1Input]
cc1 <- complete.cases(X1)
X1 <- X1[cc1,]
hirota15m1<-hirota15[cc1,]
y1 <- hirota15[cc1, "logLLNAthreshold"]
model1 <- mlpFitWithScale(X1, y1, maxit=trainIters)

##Plot training results (Reproduce Fig.2A from Hirota et al. 2015)
## ---- fig.height=8, fig.width=8, dev="png", dev.args=list(pointsize=14), echo=FALSE----
plotNetFit(model1, AssayName="EC3", topTitle="Model 1: Training Results")

##Define overlap sets in Cos Eu chemical library for model 1
euCASNotInHirota1 <- cosEu$CASRN[!cosEu$CASRN %in% hirota15m1$CASRN]
euCASInHirota1 <- cosEu$CASRN[cosEu$CASRN %in% hirota15m1$CASRN]

#############################################################################
##Model 1 Test on non-overlapping CE chemicals
cosEu1 <- cosEu[-which(cosEu$ANN_Model1_Pred=="Neg"),]
X1EUtestAllTargets <- cosEu1[euCASNotInHirota1,model1Input]
cc1eu <- complete.cases(X1EUtestAllTargets)
X1EUtestAllTargets <- X1EUtestAllTargets[cc1eu,]

##Initialize log file
ANN_model1<-X1EUtestAllTargets
ANN_model1$ANNmodel1_Set<-"Test"

## ------------------------------------------------------------------------
y1EUtestMedNegCons <- cosEu1[euCASNotInHirota1,nnTarget]
y1EUtestMedNegCons <- y1EUtestMedNegCons[cc1eu]

hirEuModel1MedNegCons <- mlpFitWithScale(X1, y1,
                                         X1EUtestAllTargets, y1EUtestMedNegCons,
                                         maxit=trainIters)
ANN_model1<-cbind(ANN_model1,hirEuModel1MedNegCons$yObs,hirEuModel1MedNegCons$yPred)

##Plot test results
## ---- fig.height=8, fig.width=8, dev="png", dev.args=list(pointsize=14), echo=FALSE----
plotNetFit(hirEuModel1MedNegCons, AssayName="EC3",
           topTitle="Model 1: Train on Hirota test on Cos EU\nMedian Neg Cons")

## ------------------------------------------------------------------------
##Write results files

X1<-cbind(X1,model1$yObs,model1$yPred)
X1$ANNmodel1_Set<-"Train"
##Overlap with CE training set
ANN_model1_train<-X1[euCASInHirota1,]

write.csv(ANN_model1_train,file=paste("ANN_Model1_TrainingSet_Obs_Pred_comp_",date.string,".csv",sep=""),row.names=T)

write.csv(ANN_model1,file=paste("ANN_Model1_TestSet_Obs_Pred_comp_",date.string,".csv",sep=""),row.names=T)

## ------------------------------------------------------------------------
##Join with larger file
cosEu[rownames(ANN_model1_train),"ANN_Model1_Pred"]<-ANN_model1_train[,"model1$yPred"]
cosEu[rownames(ANN_model1),"ANN_Model1_Pred"]<-ANN_model1[,"hirEuModel1MedNegCons$yPred"]
##Reassign negatives
cosEu$ANN_Model1_Pred[cosEu$Pos.Call.DH==0] <- "Neg"

#############################################################################
##Model 4 Training
## ------------------------------------------------------------------------
model4Input <- c("logMinhCLAT", "logDPRA", "logImax")
X4 <- hirota15[, model4Input]
cc4 <- complete.cases(X4)
X4 <- X4[cc4,]
hirota15m4<-hirota15[cc4,]
## ------------------------------------------------------------------------

y4 <- hirota15[cc4, "logLLNAthreshold"]

model4 <- mlpFitWithScale(X4, y4, maxit=trainIters)

## ---- fig.height=8, fig.width=8, dev="png", dev.args=list(pointsize=14), echo=FALSE----
plotNetFit(model4, AssayName="EC3", topTitle="Model 4: Training Results")

## ------------------------------------------------------------------------
##Define training and test sets in Cos Eu chemical library for model 4
euCASNotInHirota4 <- cosEu$CASRN[!cosEu$CASRN %in% hirota15m4$CASRN]
euCASInHirota4 <- cosEu$CASRN[cosEu$CASRN %in% hirota15m4$CASRN]

#############################################################################
##Model 4 Test
## ------------------------------------------------------------------------
cosEu4 <- cosEu[-which(cosEu$ANN_Model4_Pred=="Neg"),]
X4EUtestAllTargets <- cosEu4[euCASNotInHirota4,model4Input]
cc4euTest <- complete.cases(X4EUtestAllTargets)
X4EUtestAllTargets <- X4EUtestAllTargets[cc4euTest,]

ANN_model4<-X4EUtestAllTargets
ANN_model4$Set<-"Test"

y4EUtestMedNegCons <- cosEu4[euCASNotInHirota4,"logLLNAthresholdMedNegCons"]
y4EUtestMedNegCons <- y4EUtestMedNegCons[cc4euTest]

hirEuModel4MedNegCons <- mlpFitWithScale(X4, y4,
                                         X4EUtestAllTargets, y4EUtestMedNegCons,
                                         maxit=trainIters, stopIfOut=FALSE)
ANN_model4<-cbind(ANN_model4,hirEuModel4MedNegCons$yObs,hirEuModel4MedNegCons$yPred)

## ---- fig.height=8, fig.width=8, dev="png", dev.args=list(pointsize=14), echo=FALSE----
plotNetFit(hirEuModel4MedNegCons, AssayName="EC3",
           topTitle="Model 4: Train on Hirota test on Cos EU\nMedian Neg Cons")

## ------------------------------------------------------------------------

X4<-cbind(X4,model4$yObs,model4$yPred)
X4$Set<-"Train"
##Overlap with CE training set
ANN_model4_train<-X4[euCASInHirota4,]

write.csv(ANN_model4_train,file=paste("ANN_Model4_TrainingSet_Obs_Pred_comp",date.string,".csv",sep=""),row.names=T)

write.csv(ANN_model4,file=paste("ANN_Model4_TestSet_Obs_Pred_comp",date.string,".csv",sep=""),row.names=T)

## ------------------------------------------------------------------------
##Join with larger file
cosEu[rownames(ANN_model4_train),"ANN_Model4_Pred"]<-ANN_model4_train[,"model4$yPred"]
cosEu[rownames(ANN_model4),"ANN_Model4_Pred"]<-ANN_model4[,"hirEuModel4MedNegCons$yPred"]
##Reassign negatives
cosEu$ANN_Model4_Pred[cosEu$Pos.Call.DHK==0] <- "Neg"

#################################################################################################################
##
## END ANN CODE
##
#################################################################################################################

#Process and format ANN data for comparison with LLNA/human
##ANN Model 1
cosEu$ANN_Model1_Potency_Pred<-NA
cosEu$ANN_Model1_Potency_Pred.3class<-NA

cosEu$ANN_Model1_Pred<-as.numeric(cosEu$ANN_Model1_Pred)

#Apply potency categories to ANN predictions
cosEu$ANN_Model1_Potency_Pred[cosEu$ANN_Model1_Pred<(-1)]<-"Extreme"
cosEu$ANN_Model1_Potency_Pred[cosEu$ANN_Model1_Pred>=(-1) & as.numeric(cosEu$ANN_Model1_Pred)<0]<-"Strong"
cosEu$ANN_Model1_Potency_Pred[cosEu$ANN_Model1_Pred>=0 & as.numeric(cosEu$ANN_Model1_Pred)<1]<-"Moderate"
cosEu$ANN_Model1_Potency_Pred[cosEu$ANN_Model1_Pred>=1]<-"Weak"
cosEu$ANN_Model1_Potency_Pred[is.na(cosEu$ANN_Model1_Pred)]<-"Non"

#Change from 5 class to 3 class
cosEu$ANN_Model1_Potency_Pred.3class<-cosEu$ANN_Model1_Potency_Pred
cosEu$ANN_Model1_Potency_Pred.3class[cosEu$ANN_Model1_Potency_Pred=="Extreme"]<-"Strong"
cosEu$ANN_Model1_Potency_Pred.3class[cosEu$ANN_Model1_Potency_Pred=="Moderate"]<-"Weak"
cosEu$ANN_Model1_Potency_Pred.3class<-factor(cosEu$ANN_Model1_Potency_Pred.3class,levels=c("Non","Weak","Strong"))
#Create binary calls
cosEu$ANN_Model1_Call<-NA
cosEu$ANN_Model1_Call[!is.na(cosEu$ANN_Model1_Potency_Pred)]<-1
cosEu$ANN_Model1_Call[cosEu$ANN_Model1_Potency_Pred=="Non"]<-0

##ANN Model 4
cosEu$ANN_Model4_Potency_Pred<-NA
cosEu$ANN_Model4_Potency_Pred.3class<-NA

cosEu$ANN_Model4_Pred<-as.numeric(cosEu$ANN_Model4_Pred)

#Apply potency categories to ANN predictions
cosEu$ANN_Model4_Potency_Pred[cosEu$ANN_Model4_Pred<(-1)]<-"Extreme"
cosEu$ANN_Model4_Potency_Pred[cosEu$ANN_Model4_Pred>=(-1) & as.numeric(cosEu$ANN_Model4_Pred)<0]<-"Strong"
cosEu$ANN_Model4_Potency_Pred[cosEu$ANN_Model4_Pred>=0 & as.numeric(cosEu$ANN_Model4_Pred)<1]<-"Moderate"
cosEu$ANN_Model4_Potency_Pred[cosEu$ANN_Model4_Pred>=1]<-"Weak"
cosEu$ANN_Model4_Potency_Pred[is.na(cosEu$ANN_Model4_Pred)]<-"Non"

#Change from 5 class to 3 class
cosEu$ANN_Model4_Potency_Pred.3class<-cosEu$ANN_Model4_Potency_Pred
cosEu$ANN_Model4_Potency_Pred.3class[cosEu$ANN_Model4_Potency_Pred=="Extreme"]<-"Strong"
cosEu$ANN_Model4_Potency_Pred.3class[cosEu$ANN_Model4_Potency_Pred=="Moderate"]<-"Weak"
cosEu$ANN_Model4_Potency_Pred.3class<-factor(cosEu$ANN_Model4_Potency_Pred.3class,levels=c("Non","Weak","Strong"))
#Create binary calls
cosEu$ANN_Model4_Call<-NA
cosEu$ANN_Model4_Call[!is.na(cosEu$ANN_Model4_Potency_Pred)]<-1
cosEu$ANN_Model4_Call[cosEu$ANN_Model4_Potency_Pred=="Non"]<-0

##########################################
#Mark overlap set chemicals
cosEu$ANN_Model1_Overlap<-0
cosEu$ANN_Model1_Overlap[cosEu$CASRN %in% intersect(cosEu$CASRN,hirota15m1$CASRN)]<-1
cosEu$ANN_Model4_Overlap<-0
cosEu$ANN_Model4_Overlap[cosEu$CASRN %in% intersect(cosEu$CASRN,hirota15m4$CASRN)]<-1


write.csv(cosEu,paste("CE_invitro_invivo_predictions_withANN_",date.string,".csv",sep=""),row.names=F)

#########################################################
#Post above processing: Comparisons based on Median with Negative Consideration
#cosEu<-read.csv("CE_invitro_invivo_predictions_withANN_overlap_2017-01-08.csv",header=T)

##########################################
#Results for all chemicals (n=127) LLNA
#Create confusion matrix to compare ANN Model 1 call and LLNA result 
caret::confusionMatrix(data=cosEu$ANN_Model1_Call, reference=cosEu$LLNA_Call_MNC, positive="1")
#Create confusion matrix to compare ANN Model 1 potency class and LLNA potency result 
cosEu$LLNA_potency_3class_MNC<-factor(cosEu$LLNA_potency_3class_MNC, levels=c("Non","Weak","Strong"))
cosEu$ANN_Model1_Potency_Pred.3class<-factor(cosEu$ANN_Model1_Potency_Pred.3class, levels=c("Non","Weak","Strong"))
caret::confusionMatrix(data=cosEu$ANN_Model1_Potency_Pred.3class, reference=cosEu$LLNA_potency_3class_MNC)

#Create confusion matrix to compare ANN Model 4 call and LLNA result 
caret::confusionMatrix(data=cosEu$ANN_Model4_Call, reference=cosEu$LLNA_Call_MNC, positive="1")
#Create confusion matrix to compare ANN Model 4 potency class and LLNA potency result 
cosEu$ANN_Model4_Potency_Pred.3class<-factor(cosEu$ANN_Model4_Potency_Pred.3class, levels=c("Non","Weak","Strong"))
caret::confusionMatrix(data=cosEu$ANN_Model4_Potency_Pred.3class, reference=cosEu$LLNA_potency_3class_MNC)

##########################################
#Results for "test set" chemicals (n=53 ANN Model 1) LLNA
cosEut1<-cosEu[cosEu$CASRN %in% euCASNotInHirota1,]
#Create confusion matrix to compare ANN Model 1 call and LLNA result 
caret::confusionMatrix(data=cosEut1$ANN_Model1_Call, reference=cosEut1$LLNA_Call_MNC, positive="1")
#Create confusion matrix to compare ANN Model 1 potency class and LLNA potency result 
caret::confusionMatrix(data=cosEut1$ANN_Model1_Potency_Pred.3class, reference=cosEut1$LLNA_potency_3class_MNC)

#Results for "test set" chemicals (n=83 ANN Model 4) LLNA
cosEut4<-cosEu[cosEu$CASRN %in% euCASNotInHirota4,]
#Create confusion matrix to compare ANN Model 4 call and LLNA result 
caret::confusionMatrix(data=cosEut4$ANN_Model4_Call, reference=cosEut4$LLNA_Call_MNC, positive="1")
#Create confusion matrix to compare ANN Model 4 potency class and LLNA potency result 
caret::confusionMatrix(data=cosEut4$ANN_Model4_Potency_Pred.3class, reference=cosEut4$LLNA_potency_3class_MNC)

##########################################
#Results for all chemicals (n=127) Human
#Create confusion matrix to compare ANN Model 1 call and Human result 
caret::confusionMatrix(data=cosEu$ANN_Model1_Call, reference=cosEu$human_Call, positive="1")
#Create confusion matrix to compare ANN Model 1 potency class and human potency result 
cosEu$human_potency_3class<-factor(cosEu$human_potency_3class, levels=c("Non","Weak","Strong"))
caret::confusionMatrix(data=cosEu$ANN_Model1_Potency_Pred.3class, reference=cosEu$human_potency_3class)

#Create confusion matrix to compare ANN Model 4 call and human result 
caret::confusionMatrix(data=cosEu$ANN_Model4_Call, reference=cosEu$human_Call, positive="1")
#Create confusion matrix to compare ANN Model 4 potency class and human potency result 
caret::confusionMatrix(data=cosEu$ANN_Model4_Potency_Pred.3class, reference=cosEu$human_potency_3class)

##########################################
#Results for "test set" chemicals (n=53 ANN Model 1) human
cosEut1<-cosEu[cosEu$CASRN %in% euCASNotInHirota1,]
#Create confusion matrix to compare ANN Model 1 call and human result 
caret::confusionMatrix(data=cosEut1$ANN_Model1_Call, reference=cosEut1$human_Call, positive="1")
#Create confusion matrix to compare ANN Model 1 potency class and human potency result 
caret::confusionMatrix(data=cosEut1$ANN_Model1_Potency_Pred.3class, reference=cosEut1$human_potency_3class)

#Results for "test set" chemicals (n=83 ANN Model 4) human
cosEut4<-cosEu[cosEu$CASRN %in% euCASNotInHirota4,]
#Create confusion matrix to compare ANN Model 4 call and human result 
caret::confusionMatrix(data=cosEut4$ANN_Model4_Call, reference=cosEut4$human_Call, positive="1")
#Create confusion matrix to compare ANN Model 4 potency class and human potency result 
caret::confusionMatrix(data=cosEut4$ANN_Model4_Potency_Pred.3class, reference=cosEut4$human_potency_3class)

